//******************************************************************************
//Name          : FrequencyResponse.cs
//Description   : Implementation to experience the beauty and the pitfalls Filtering
//Author        : Dr. Oliver Kluge, 90411 Nuernberg, Germany
//Copyright(C)  : Oliver Kluge 2014 
//------------------------------------------------------------------------------
//******************************************************************************

using System;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;
using Display;


class Filter
{
    const int samplerate = 128;                     // Filterlnge
    const double PI       =   3.141592653589793;

    public double[] a = new double[samplerate];     // Filterkoeffizienten des Vorwrtszweiges
    public double[] b = new double[samplerate];	    // Filterkoeffizienten des Rckkopplungszweiges
    public double[] An = new double[samplerate];    // Amplitudengang des Zhlers von H(z)
    public double[] Ad = new double[samplerate];    // Amplitudengang des Nenners von H(z)
    public double[] Pn = new double[samplerate];    // Phasengang des Zhlers von H(z)
    public double[] Pd = new double[samplerate];    // Phasengang des Nenners von H(z)
    public double[] AS = new double[samplerate];    // Amplitudenspektrum von H(z)
    public double[] PS = new double[samplerate];    // Phasenspektrum von H(z)

    public int Samplerate
    {
        get { return samplerate; }
    }

    public void CreateTestsignal(int select)
    {
        // FIR-Filter
        if (select == 1)
        {
            int M = 10;

            for (int k = 0; k < samplerate; k++)
            {
                if (k < M)
                {
                    a[k] = 1.0/(double)M;
                }
            }
        }

        // IIR-Filter
        if (select == 2)
        {
            // Filterkoeffizienten des Vorwrtszweiges Zhler{H(z)} = a0 + a1*z^-1 + a2*z^-2 + ...
            a[0] = 1;
            a[1] = 0;
            a[2] = 0;

            // Filterkoeffizienten des Rckkopplungszweiges Nenner{H(z)} = 1 - b1*z^-1 - b2*z^-2 - ...
            b[0] = 1;       // Fixwert!!!
            //b[1] = -1;
            //b[2] = 0.25;
            b[1] = -2;
            b[2] = 2;
            b[3] = -1;
            b[4] = 0.25;
        }
    }

    public void FrequencyResponse(double startFreq, double stopFreq)
    {
        double deltaf = (stopFreq-startFreq)/(samplerate);
        double freq   = startFreq;
        double real   = 0;
        double imag   = 0;

        for (int i=0; i<samplerate; i++)
        {
            real = 0;
            imag = 0;

            // compute all coefficients a[k] of the forward path
            for (int k=0; k<samplerate; k++)
            {
                real += a[k]*Math.Cos(2*PI*freq*k/samplerate);
                imag += a[k]*Math.Sin(2*PI*freq*k/samplerate);
            }

            // amplitude and phase of the forward path -> numerator of H(z)
            An[i] = Math.Sqrt(real*real + imag*imag);
            Pn[i] = Math.Atan2(imag, real);

            real = 0;
            imag = 0;

            // compute all coefficients b[k] of the feedback path
            for (int k=0; k<samplerate; k++)
            {
	            real += b[k]*Math.Cos(2*PI*freq*k/samplerate);
	            imag += b[k]*Math.Sin(2*PI*freq*k/samplerate);
            }

            // amplitude and phase of the feedback path -> denominator of H(z)
            Ad[i] = Math.Sqrt(real*real + imag*imag);
            Pd[i] = Math.Atan2(imag, real);

            // amplitude and phase of the overall system -> H(z) = Hn(z) / Hd(z)
            if (Ad[i] > 0)
            {
                // IIR-Filter
                AS[i] = An[i] / Ad[i];
            }
            else
            {
                // FIR-Filter
                AS[i] = An[i];
            }
            PS[i] = Pn[i] - Pd[i];

            freq += deltaf;
        }
    }
}


class MainApp
{
    [STAThread]
    static void Main()
    {
        string userinput = "";
        int select = 0;
        double freqStart = 0;
        double freqStop  = 0;
        Filter myFilter = new Filter();

        Console.Write("Calculate the filter's frequency response from its coefficients\n");

        Console.Write("Mittelwertfilter -> 1\n");
        Console.Write("IIR-Filter       -> 2\n");
        userinput = Console.ReadLine();
        select = Convert.ToInt32(userinput);
        myFilter.CreateTestsignal(select);

        //Console.WriteLine();
        //Console.Write("Startfrequenz: ");
        //userinput = Console.ReadLine();
        //freqStart = Convert.ToDouble(userinput);
        //Console.Write("Endfrequenz  : ");
        //userinput = Console.ReadLine();
        //freqStop = Convert.ToDouble(userinput);
        Console.WriteLine();

        myFilter.FrequencyResponse(0, myFilter.Samplerate/2);

        double[] dArg = new double[myFilter.Samplerate];  // double argument to display frequency response
        for (int k = 0; k < myFilter.Samplerate; k++)
        {
            dArg[k] = k / 2.0;
        }

        DisplayChart mydisplay = new DisplayChart(720, 480, SeriesChartType.Line, "Frequency response", "f", "|H(f)|");
        mydisplay.AddData(dArg, myFilter.AS);

        // ...and run the chart-application
        Application.EnableVisualStyles();
        Application.Run(mydisplay);

        mydisplay.Dispose();
        Application.Exit();

    }
}
